// Copyright 2012, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Author: Hannah Bast <bast>.


/**
 * Class realizing a doubly linked list. Items of the list are objects of the
 * class LinkedListItem; see LinkedListItem.java.
 *
 * Note: For demonstration purposes we only implement the insertAfter method.
 * For a LinkedList class that is usable in practice, there should also be a
 * remove method.
 * 
 * @param <T> The type of the elements in the list.
 */
public class LinkedList<T> {

  /**
   * Create an empty list.
   */
  public LinkedList() {
    first = null;
    last = null;
    size = 0;
  }

  /**
   * Insert a new item after the given item. If the given item is null,
   * insert the new item at the beginning of the list.
   */
  public void insertAfter(LinkedListItem<T> previousItem, 
    LinkedListItem<T> newItem) {    
    LinkedListItem<T> nextItem;
    // Check if we insert in the beginning.
    if (previousItem == null) {
      nextItem = first;
      first = newItem;
    } else {
      nextItem = previousItem.next;
      previousItem.next = newItem;
    }
    
    // Check if we inserted the last item
    if (nextItem == null) {
      last = newItem;
    } else {
      nextItem.previous = newItem;
    }
    
    newItem.previous = previousItem;
    newItem.next = nextItem;
    ++size;
  }

  /** 
   * The list in human-readable form (for debugging).
   */
  public String toString() {
    StringBuilder sb = new StringBuilder();
    sb.append("first = " + LinkedListItem.id(first) + "\n");
    LinkedListItem<T> item = first;
    while (item != null) {
      sb.append(item.toString());
      sb.append("\n");
      item = item.next;
    }
    sb.append("last = " + LinkedListItem.id(last) + "\n");
    return sb.toString();
  }

  /**
   * Getter for first.
   */
  public LinkedListItem<T> first() { return first; }

  /**
   * Getter for last.
   */
  public LinkedListItem<T> last() { return last; }

  /**
   * Getter for size.
   */
  public int size() { return size; }

  /**
   * Pointer to the first list item; is null, if and only if list is empty.
   */
  private LinkedListItem<T> first;

  /**
   * Pointer to the last list item; is null, if and only if list is empty.
   */
  private LinkedListItem<T> last;

  /**
   * The number of items in the list. Zero, if list is empty.
   */
  private int size;
}
